/*
	RT Bench - Benchmarks various hash functions for Rainbow Tables
	Copyright (C) 2012 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "hashfactory.h"
#include "cisco.h"
#include "ciscohelp.h"
#include "md5.h"
#include "md5help.h"

uint32_t Cisco::inited = Cisco::initCalc();

#ifdef ARC_x86

void Cisco::calcSSE2(vector4 *pw, vector4 *hash)
{
	if (__MD5_INTERLACED__ > 0) { ((__m128i*) pw)[0 * __MD5_INTERLACED__ + 0] = _mm_set1_epi32(16); }
	if (__MD5_INTERLACED__ > 1) { ((__m128i*) pw)[0 * __MD5_INTERLACED__ + 1] = _mm_set1_epi32(16); }
	if (__MD5_INTERLACED__ > 2) { ((__m128i*) pw)[0 * __MD5_INTERLACED__ + 2] = _mm_set1_epi32(16); }
	if (__MD5_INTERLACED__ > 3) { ((__m128i*) pw)[0 * __MD5_INTERLACED__ + 3] = _mm_set1_epi32(16); }
	if (__MD5_INTERLACED__ > 0) { ((__m128i*) pw)[5 * __MD5_INTERLACED__ + 0] = _mm_set1_epi32(0x80); }
	if (__MD5_INTERLACED__ > 1) { ((__m128i*) pw)[5 * __MD5_INTERLACED__ + 1] = _mm_set1_epi32(0x80); }
	if (__MD5_INTERLACED__ > 2) { ((__m128i*) pw)[5 * __MD5_INTERLACED__ + 2] = _mm_set1_epi32(0x80); }
	if (__MD5_INTERLACED__ > 3) { ((__m128i*) pw)[5 * __MD5_INTERLACED__ + 3] = _mm_set1_epi32(0x80); }
	MD5::calcSSE2(pw, hash);
	if (__MD5_INTERLACED__ > 0) { FINISH_CISCO(hash, 0); }
	if (__MD5_INTERLACED__ > 1) { FINISH_CISCO(hash, 1); }
	if (__MD5_INTERLACED__ > 2) { FINISH_CISCO(hash, 2); }
	if (__MD5_INTERLACED__ > 3) { FINISH_CISCO(hash, 3); }
}

#endif

void Cisco::calcNorm(vector4 *pw, vector4 *hash)
{
	uint32_t hash0, hash1, hash2, hash3;

	pw[0].data[0] = 16;
	pw[5].data[0] = TO_LITTLE_ENDIAN_32(0x80);
	MD5::calcNorm(pw, hash);
	hash0 = FROM_LITTLE_ENDIAN_32(hash[0].data[0]);
	hash1 = FROM_LITTLE_ENDIAN_32(hash[1].data[0]);
	hash2 = FROM_LITTLE_ENDIAN_32(hash[2].data[0]);
	hash3 = FROM_LITTLE_ENDIAN_32(hash[3].data[0]);
	hash[0].data[0] = TO_LITTLE_ENDIAN_32(( hash0        & 0x00ffffff) | (hash1 << 24));
	hash[1].data[0] = TO_LITTLE_ENDIAN_32(((hash1 >>  8) & 0x0000ffff) | (hash2 << 16));
	hash[2].data[0] = TO_LITTLE_ENDIAN_32(((hash2 >> 16) & 0x000000ff) | (hash3 <<  8));
}

uint32_t Cisco::initCalc()
{
#ifdef ARC_x86
	if ((getInstructionSets() & (IS_AVX | IS_XOP)) == (IS_AVX | IS_XOP))
	{
		return HashFactory::registerHash("cisco", QUOTE(__MD5_INTERLACED__) "xAVX_XOP", 0, 16, Cisco::calcAVXXOP,  12, 4 * __MD5_INTERLACED__, CE_ASCII_BIN8 | CE_NULL_PADDED, 5, 1);
	}
	else if ((getInstructionSets() & IS_AVX) == IS_AVX)
	{
		return HashFactory::registerHash("cisco", QUOTE(__MD5_INTERLACED__) "xAVX",     0, 16, Cisco::calcAVX,     12, 4 * __MD5_INTERLACED__, CE_ASCII_BIN8 | CE_NULL_PADDED, 5, 1);
	}
	else if ((getInstructionSets() & IS_SSE2) == IS_SSE2)
	{
		return HashFactory::registerHash("cisco", QUOTE(__MD5_INTERLACED__) "xSSE2",    0, 16, Cisco::calcSSE2,    12, 4 * __MD5_INTERLACED__, CE_ASCII_BIN8 | CE_NULL_PADDED, 5, 1);
	}
#endif
	return HashFactory::registerHash("cisco", "scalar",                             0, 16, Cisco::calcNorm,    12,                      1, CE_ASCII_BIN8 | CE_NULL_PADDED, 5, 1);
}
