/*
	RT Bench - Benchmarks various hash functions for Rainbow Tables
	Copyright (C) 2012 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "hashfactory.h"
#include "doublebinarymd5.h"
#include "md5.h"
#include "md5help.h"

uint32_t DoubleBinaryMD5::inited = DoubleBinaryMD5::initCalc();

#ifdef ARC_x86

void DoubleBinaryMD5::calcSSE2(vector4 *pw, vector4 *hash)
{
	MD5::calcSSE2(pw, hash + __MD5_INTERLACED__);
	if (__MD5_INTERLACED__ > 0) { ((__m128i*) hash)[0 * __MD5_INTERLACED__ + 0] = _mm_set1_epi32(16); }
	if (__MD5_INTERLACED__ > 1) { ((__m128i*) hash)[0 * __MD5_INTERLACED__ + 1] = _mm_set1_epi32(16); }
	if (__MD5_INTERLACED__ > 2) { ((__m128i*) hash)[0 * __MD5_INTERLACED__ + 2] = _mm_set1_epi32(16); }
	if (__MD5_INTERLACED__ > 3) { ((__m128i*) hash)[0 * __MD5_INTERLACED__ + 3] = _mm_set1_epi32(16); }
	if (__MD5_INTERLACED__ > 0) { ((__m128i*) hash)[5 * __MD5_INTERLACED__ + 0] = _mm_set1_epi32(0x80); }
	if (__MD5_INTERLACED__ > 1) { ((__m128i*) hash)[5 * __MD5_INTERLACED__ + 1] = _mm_set1_epi32(0x80); }
	if (__MD5_INTERLACED__ > 2) { ((__m128i*) hash)[5 * __MD5_INTERLACED__ + 2] = _mm_set1_epi32(0x80); }
	if (__MD5_INTERLACED__ > 3) { ((__m128i*) hash)[5 * __MD5_INTERLACED__ + 3] = _mm_set1_epi32(0x80); }
	MD5::calcSSE2(hash, hash);
}

#endif

void DoubleBinaryMD5::calcNorm(vector4 *pw, vector4 *hash)
{
	MD5::calcNorm(pw, hash+1);
	hash[0].data[0] = 16;
	hash[5].data[0] = TO_LITTLE_ENDIAN_32(0x80);
	MD5::calcNorm(hash, hash);
}

uint32_t DoubleBinaryMD5::initCalc()
{
#ifdef ARC_x86
	if ((getInstructionSets() & (IS_AVX | IS_XOP)) == (IS_AVX | IS_XOP))
	{
		return HashFactory::registerHash("doublebinmd5", QUOTE(__MD5_INTERLACED__) "xAVX_XOP", 0, 19, DoubleBinaryMD5::calcAVXXOP,  16, 4 * __MD5_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 5, 2);
	}
	else if ((getInstructionSets() & IS_AVX) == IS_AVX)
	{
		return HashFactory::registerHash("doublebinmd5", QUOTE(__MD5_INTERLACED__) "xAVX",     0, 19, DoubleBinaryMD5::calcAVX,     16, 4 * __MD5_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 5, 2);
	}
	else if ((getInstructionSets() & IS_SSE2) == IS_SSE2)
	{
		return HashFactory::registerHash("doublebinmd5", QUOTE(__MD5_INTERLACED__) "xSSE2",    0, 19, DoubleBinaryMD5::calcSSE2,    16, 4 * __MD5_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 5, 2);
	}
#endif
	return HashFactory::registerHash("doublebinmd5", "scalar",                             0, 19, DoubleBinaryMD5::calcNorm,    16,                      1, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 5, 2);
}
