/*
	RT Bench - Benchmarks various hash functions for Rainbow Tables
	Copyright (C) 2012 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <stdio.h>
#include <string.h>
#include "hashfactory.h"
#include "keyspace.h"

#define QUAD(ch) (((uint32_t)(ch)) + (((uint32_t)(ch)) << 8) + (((uint32_t)(ch)) << 16) + (((uint32_t)(ch)) << 24))
void benchmark(KeySpace &ks, hashAlgorithm hashAlgo, uint32_t tableIndex);

int main()
{
	hashAlgorithm algo;
	vector4 *pw, *hash, *deleteMe;
	KeySpace *ks;
	uint32_t vecSize, numPwBlocks, numHashBlocks, i, j, k;
	list<list<string> > subKeySpaces;
	list<string> charSets;
	string charSet = "abcdefghijklmnopqrstuvwxyz";
	char buffer[1024], tmpBuffer[64]; // 661 max, 42 max

	charSets.push_back(charSet); // length 1
	subKeySpaces.push_back(charSets);
	charSets.push_back(charSet); // length 2
	subKeySpaces.push_back(charSets);
	charSets.push_back(charSet); // length 3
	subKeySpaces.push_back(charSets);
	charSets.push_back(charSet); // length 4
	subKeySpaces.push_back(charSets);
	charSets.push_back(charSet); // length 5
	subKeySpaces.push_back(charSets);
	charSets.push_back(charSet); // length 6
	subKeySpaces.push_back(charSets);
	charSets.push_back(charSet); // length 7
	subKeySpaces.push_back(charSets);

	printf("Supported hashing algorithms:\n");
	HashFactory::printHashes();


	// *******************************
	// *** Test Reduction Function ***
	// *******************************
	printf("Testing Reduction Function... ");
	fflush(stdout);
	// md5_loweralpha#1-7_0_2400x20_0.rt
	uint32_t characterEncoding;
	uint64_t sp[20] = {
		UINT64_C(0x00000000BC8B7A03), UINT64_C(0x00000001CCD03F68), UINT64_C(0x00000000F49FE149), UINT64_C(0x000000014934CEE9), 
		UINT64_C(0x00000001540061C9), UINT64_C(0x00000000CAB7CF72), UINT64_C(0x000000015244D252), UINT64_C(0x00000001493EFE25), 
		UINT64_C(0x00000000B9A845A6), UINT64_C(0x00000000BE8E3E44), UINT64_C(0x00000001B4AF7DCC), UINT64_C(0x00000001D52C2173), 
		UINT64_C(0x00000001EE935191), UINT64_C(0x000000016E4EB81D), UINT64_C(0x00000001EE4CA4D0), UINT64_C(0x000000013A0B74D5), 
		UINT64_C(0x0000000121F39670), UINT64_C(0x00000000C72A1C9A), UINT64_C(0x00000000149482D6), UINT64_C(0x00000001661B467B)};
	uint64_t ep[20] = {
		UINT64_C(0x000000000D3EF565), UINT64_C(0x000000001E211D9B), UINT64_C(0x0000000028AAC4E1), UINT64_C(0x00000000486B8A70), 
		UINT64_C(0x000000004E97B03F), UINT64_C(0x000000006E51199A), UINT64_C(0x000000007BC3CDBD), UINT64_C(0x0000000090A6FE74), 
		UINT64_C(0x00000000AFCB7BBB), UINT64_C(0x00000000C2FFEFE6), UINT64_C(0x00000000EDA1057A), UINT64_C(0x0000000103541090), 
		UINT64_C(0x000000010ABE1E23), UINT64_C(0x000000012BC4D3AA), UINT64_C(0x0000000167DFC777), UINT64_C(0x0000000190018571), 
		UINT64_C(0x00000001A13E3907), UINT64_C(0x00000001A6D878D2), UINT64_C(0x00000001AF98F69F), UINT64_C(0x00000001D91C9B3A)};

	characterEncoding = HashFactory::getHashCharacterEncoding("md5");
	if (characterEncoding == 0)
	{
		printf("No MD5 algorithm\n");
	}
	else
	{
		ks = new KeySpace(subKeySpaces, characterEncoding);

		if (HashFactory::getHashInstance("md5", ks->getMinPwLen(), ks->getMaxPwLen(), algo))
		{
			printf("No MD5 algorithm\n");
		}
		else
		{
			uint64u pwNum;
			uint64_t keySpace;
			uint64_t *pwNums;

			pwNums = new uint64_t[algo.hashesPerCall];
			vecSize = (algo.hashesPerCall + 3) / 4;
			allocatePwHash(&pw, &hash, &deleteMe, vecSize, algo.numPwBlocks, (algo.len + 3) / 4 + algo.numExtraHashBlocks);

			keySpace = ks->getKeySpace();
			for (i = 0; i < algo.hashesPerCall; i++)
			{
				pwNums[i] = sp[i];
			}
			ks->get(pw, algo.hashesPerCall, vecSize, pwNums);
			for (i = 0; i < 2398; i++)
			{
				algo.func(pw, hash);
				for (j = 0; j < algo.hashesPerCall; j++)
				{
					pwNum.lo32.u32 = FROM_LITTLE_ENDIAN_32(hash[0].data[j]);
					pwNum.hi32.u32 = FROM_LITTLE_ENDIAN_32(hash[vecSize].data[j]);
					pwNums[j] = (pwNum.u64 + (0 + i)) % keySpace;
				}
				ks->get(pw, algo.hashesPerCall, vecSize, pwNums);
			}
			algo.func(pw, hash);
			for (j = 0; j < algo.hashesPerCall; j++)
			{
				pwNum.lo32.u32 = FROM_LITTLE_ENDIAN_32(hash[0].data[j]);
				pwNum.hi32.u32 = FROM_LITTLE_ENDIAN_32(hash[vecSize].data[j]);
				pwNums[j] = (pwNum.u64 + (0 + i)) % keySpace;
			}
			delete [] deleteMe;
			for (i = 0; i < algo.hashesPerCall; i++)
			{
				if (pwNums[i] != ep[i])
				{
					break;
				}
			}
			delete [] pwNums;
			if (i == algo.hashesPerCall)
			{
				printf("PASS\n");
			}
			else
			{
				printf("FAIL\n");
			}
		}
		delete ks;
	}


	// ******************
	// *** Test Cisco ***
	// ******************
	printf("Testing Cisco...              ");
	fflush(stdout);
	if (HashFactory::getHashInstance("cisco", 0, 16, algo))
	{
		printf("No Cisco algorithm\n");
	}
	else
	{
		vecSize = (algo.hashesPerCall + 3) / 4;
		numPwBlocks = algo.numPwBlocks;
		numHashBlocks = (algo.len + 3) / 4 + algo.numExtraHashBlocks;
		allocatePwHash(&pw, &hash, &deleteMe, vecSize, numPwBlocks, numHashBlocks);
		for (i = 0; i < vecSize; i++)
		{
			pw[0*vecSize+i].data[0] = (numPwBlocks-1)*4-1;
			pw[0*vecSize+i].data[1] = (numPwBlocks-1)*4-1;
			pw[0*vecSize+i].data[2] = (numPwBlocks-1)*4-1;
			pw[0*vecSize+i].data[3] = (numPwBlocks-1)*4-1;
		}
		for (i = 1; i < numPwBlocks; i++)
		{
			for (j = 0; j < vecSize; j++)
			{
				pw[i*vecSize+j].data[0] = QUAD(' ' + (i-1)+4*(numPwBlocks-1)*j+0*(numPwBlocks-1));
				pw[i*vecSize+j].data[1] = QUAD(' ' + (i-1)+4*(numPwBlocks-1)*j+1*(numPwBlocks-1));
				pw[i*vecSize+j].data[2] = QUAD(' ' + (i-1)+4*(numPwBlocks-1)*j+2*(numPwBlocks-1));
				pw[i*vecSize+j].data[3] = QUAD(' ' + (i-1)+4*(numPwBlocks-1)*j+3*(numPwBlocks-1));
			}
		}
		algo.func(pw, hash);
		buffer[0] = 0;
		for (j = 0; j < vecSize; j++)
		{
			for (k = 0; k < 4; k++)
			{
				sprintf(tmpBuffer, "%08x%08x%08x\n", TO_BIG_ENDIAN_32(hash[0*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[1*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[2*vecSize+j].data[k]));
				strcat(buffer, tmpBuffer);
			}
		}
		delete [] deleteMe;
		if (memcmp(buffer,
			"07437f6499e1cf214b6e2225\n"
			"0d0b65b50ce4f1a59199ac0e\n"
			"1f1fba17ff6fb0952f9df028\n"
			"6151c127d071263535c58a8e\n"
			"4eb3475e7a6bcd5f64b3883a\n"
			"82121f56e8dbfac168456a8b\n"
			"1f8164ecc5ff61de43cd8043\n"
			"7b219330ff08e0db3e0ed8cb\n"
			"7c27897877eb5856a9f1de4f\n"
			"0208204f77a3d412ca354e6a\n"
			"aab3b19cd3f31d3c6b5e79c9\n"
			"86212713a5858308d0e01908\n"
			"9800a131c6664b5e1fe400fc\n"
			"bb4fb208cddef495ee65f4a6\n"
			"3d79d4ffe9c1286cb0c2c181\n"
			"50fdc455edab8e9908508bde\n", strlen(buffer)) == 0)

		{
			printf("PASS\n");
		}
		else
		{
			printf("FAIL\n");
		}
		ks = new KeySpace(subKeySpaces, algo.charaterEncoding);
		benchmark(*ks, algo, 0);
		delete ks;
	}


	// ****************
	// *** Test MD5 ***
	// ****************
	printf("Testing MD5...                ");
	fflush(stdout);
	if (HashFactory::getHashInstance("md5", 0, 19, algo))
	{
		printf("No MD5 algorithm\n");
	}
	else
	{
		vecSize = (algo.hashesPerCall + 3) / 4;
		numPwBlocks = algo.numPwBlocks;
		numHashBlocks = (algo.len + 3) / 4 + algo.numExtraHashBlocks;
		allocatePwHash(&pw, &hash, &deleteMe, vecSize, numPwBlocks, numHashBlocks);
		for (i = 0; i < vecSize; i++)
		{
			pw[0*vecSize+i].data[0] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[1] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[2] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[3] = numPwBlocks*4-1;
		}
		for (i = 1; i < numPwBlocks; i++)
		{
			for (j = 0; j < vecSize; j++)
			{
				pw[i*vecSize+j].data[0] = QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks);
				pw[i*vecSize+j].data[1] = QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks);
				pw[i*vecSize+j].data[2] = QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks);
				pw[i*vecSize+j].data[3] = QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks);
			}
		}
		for (j = 0; j < vecSize; j++)
		{
			pw[i*vecSize+j].data[0] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[1] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[2] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[3] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks) & 0x00ffffff) + 0x80000000);
		}
		algo.func(pw, hash);
		buffer[0] = 0;
		for (j = 0; j < vecSize; j++)
		{
			for (k = 0; k < 4; k++)
			{
				sprintf(tmpBuffer, "%08x%08x%08x%08x\n", TO_BIG_ENDIAN_32(hash[0*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[1*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[2*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[3*vecSize+j].data[k]));
				strcat(buffer, tmpBuffer);
			}
		}
		delete [] deleteMe;
		if (memcmp(buffer,
			"04db24a66530638ce872cb861c5fd63a\n"
			"f3e1abf74ec4e1a43426e6ab31f2dd71\n"
			"09d164a6668bf4a0c616f885ecd65cb0\n"
			"133ca337e185686480d00beb845976df\n"
			"d30d93da449f8c56e5edc9b7326ddbba\n"
			"73276997bbd51eebc9c1b27401aa5d21\n"
			"2d6c55c2a705ba9880c6ffba5baeb70e\n"
			"7964915d987626570d8c1f8bcaba962c\n"
			"bfd88378ca2edf62464aad324098d0ee\n"
			"bcaae53cc4c3b16825d26b9f028ca4ba\n"
			"0a28c56d1d98ac8d52de67a7103f6b19\n"
			"93d23734dcd65ff31ae98acc86ae26d8\n"
			"67e7af2c4cb0d5fe3a2ddd1acc2460a9\n"
			"1981bbda8bd51495693b57cfd9e95866\n"
			"9b9ad81ccede50492fd7fa9cbdf6ed32\n"
			"85f0b0bfe46e5050362dbf5c058aad93\n", strlen(buffer)) == 0)
		{
			printf("PASS\n");
		}
		else
		{
			printf("FAIL\n");
		}
		ks = new KeySpace(subKeySpaces, algo.charaterEncoding);
		benchmark(*ks, algo, 0);
		delete ks;
	}


	// ******************************
	// *** Test Double Binary MD5 ***
	// ******************************
	printf("Testing Double Binary MD5...  ");
	fflush(stdout);
	if (HashFactory::getHashInstance("doublebinmd5", 0, 19, algo))
	{
		printf("No Double Binary MD5 algorithm\n");
	}
	else
	{
		vecSize = (algo.hashesPerCall + 3) / 4;
		numPwBlocks = algo.numPwBlocks;
		numHashBlocks = (algo.len + 3) / 4 + algo.numExtraHashBlocks;
		allocatePwHash(&pw, &hash, &deleteMe, vecSize, numPwBlocks, numHashBlocks);
		for (i = 0; i < vecSize; i++)
		{
			pw[0*vecSize+i].data[0] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[1] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[2] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[3] = numPwBlocks*4-1;
		}
		for (i = 1; i < numPwBlocks; i++)
		{
			for (j = 0; j < vecSize; j++)
			{
				pw[i*vecSize+j].data[0] = QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks);
				pw[i*vecSize+j].data[1] = QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks);
				pw[i*vecSize+j].data[2] = QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks);
				pw[i*vecSize+j].data[3] = QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks);
			}
		}
		for (j = 0; j < vecSize; j++)
		{
			pw[i*vecSize+j].data[0] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[1] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[2] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[3] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks) & 0x00ffffff) + 0x80000000);
		}
		algo.func(pw, hash);
		buffer[0] = 0;
		for (j = 0; j < vecSize; j++)
		{
			for (k = 0; k < 4; k++)
			{
				sprintf(tmpBuffer, "%08x%08x%08x%08x\n", TO_BIG_ENDIAN_32(hash[0*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[1*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[2*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[3*vecSize+j].data[k]));
				strcat(buffer, tmpBuffer);
			}
		}
		delete [] deleteMe;
		if (memcmp(buffer,
			"b963f4248cb9b311de16349fa1df4f2d\n"
			"66c336ac60ff8f0127534eef51e71539\n"
			"ec6ed25145ea253f2754c35b68a77d01\n"
			"6b88c4044f371ab83becbc309065e1e1\n"
			"b509a3f98726538ae85c2cea9780da09\n"
			"747fcb442fda31182129b28e811491b8\n"
			"323f634f1dab926d7cafbbb86985e395\n"
			"8fbea9a91a8cc00cc17bdb95a9ba187f\n"
			"b7e4352bb82631d1dc7ea528b3a90b3e\n"
			"5f73b697be5853abfad4745f7a0d71a5\n"
			"86217fd03d20a567f04695414bfbe895\n"
			"c955ad55987bbeb429319b3ec73238f4\n"
			"2abb3230f4ca07013b37f8f92f769664\n"
			"f30badf13ea9d767eef47251896e4d10\n"
			"15979cf885994e08429f8a9c18e39029\n"
			"21f127d1e58f71a212fed06e02d2f8f6\n", strlen(buffer)) == 0)
		{
			printf("PASS\n");
		}
		else
		{
			printf("FAIL\n");
		}
		ks = new KeySpace(subKeySpaces, algo.charaterEncoding);
		benchmark(*ks, algo, 0);
		delete ks;
	}


	// ***********************
	// *** Test Double MD5 ***
	// ***********************
	printf("Testing Double MD5...         ");
	fflush(stdout);
	if (HashFactory::getHashInstance("doublemd5", 0, 19, algo))
	{
		printf("No Double MD5 algorithm\n");
	}
	else
	{
		vecSize = (algo.hashesPerCall + 3) / 4;
		numPwBlocks = algo.numPwBlocks;
		numHashBlocks = (algo.len + 3) / 4 + algo.numExtraHashBlocks;
		allocatePwHash(&pw, &hash, &deleteMe, vecSize, numPwBlocks, numHashBlocks);
		for (i = 0; i < vecSize; i++)
		{
			pw[0*vecSize+i].data[0] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[1] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[2] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[3] = numPwBlocks*4-1;
		}
		for (i = 1; i < numPwBlocks; i++)
		{
			for (j = 0; j < vecSize; j++)
			{
				pw[i*vecSize+j].data[0] = QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks);
				pw[i*vecSize+j].data[1] = QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks);
				pw[i*vecSize+j].data[2] = QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks);
				pw[i*vecSize+j].data[3] = QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks);
			}
		}
		for (j = 0; j < vecSize; j++)
		{
			pw[i*vecSize+j].data[0] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[1] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[2] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[3] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks) & 0x00ffffff) + 0x80000000);
		}
		algo.func(pw, hash);
		buffer[0] = 0;
		for (j = 0; j < vecSize; j++)
		{
			for (k = 0; k < 4; k++)
			{
				sprintf(tmpBuffer, "%08x%08x%08x%08x\n", TO_BIG_ENDIAN_32(hash[0*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[1*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[2*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[3*vecSize+j].data[k]));
				strcat(buffer, tmpBuffer);
			}
		}
		delete [] deleteMe;
		if (memcmp(buffer,
			"fd01c7cd3dc18d19c168a417e22d77ee\n"
			"5b7ab8d8fdee966a853241461729727a\n"
			"2eec111b36c37e46174c0f4eeaa3014a\n"
			"2b5062b7db42d45309099121560409e9\n"
			"dfa0c20ec1950a62bf3e0c6edad1aad8\n"
			"46af9b3ca5b9aecaf1b43862170b5864\n"
			"932cb4dfd994fc54686df638dadecb34\n"
			"5a5ad93567fb2b86f646963031dda489\n"
			"8aab5727b3b609c2a3c1adb2c30f2543\n"
			"8863f28d187c6f4d2db25204b34bd9e0\n"
			"9f4d4dbbc4a11f37a4751ef015eac106\n"
			"ce8800bb06576bb9fabc6af69563f8d1\n"
			"5fdd6392efbfe0c3cd3fa29b37babcec\n"
			"6fe830d0be136d6b1c9cddcd888bc6f5\n"
			"0e5cd7a348b0f0d29d89e3b03dff2561\n"
			"8287297f1b988f02a16f79d2dde4bc60\n", strlen(buffer)) == 0)
		{
			printf("PASS\n");
		}
		else
		{
			printf("FAIL\n");
		}
		ks = new KeySpace(subKeySpaces, algo.charaterEncoding);
		benchmark(*ks, algo, 0);
		delete ks;
	}


	// *********************
	// *** Test MD4/NTLM ***
	// *********************
	printf("Testing MD4/NTLM...           ");
	fflush(stdout);
	if (HashFactory::getHashInstance("md4", 0, 31, algo))
	{
		printf("No MD4 algorithm\n");
	}
	else
	{
		vecSize = (algo.hashesPerCall + 3) / 4;
		numPwBlocks = algo.numPwBlocks;
		numHashBlocks = (algo.len + 3) / 4 + algo.numExtraHashBlocks;
		allocatePwHash(&pw, &hash, &deleteMe, vecSize, numPwBlocks, numHashBlocks);
		for (i = 0; i < vecSize; i++)
		{
			pw[0*vecSize+i].data[0] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[1] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[2] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[3] = numPwBlocks*4-1;
		}
		for (i = 1; i < numPwBlocks; i++)
		{
			for (j = 0; j < vecSize; j++)
			{
				pw[i*vecSize+j].data[0] = QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks);
				pw[i*vecSize+j].data[1] = QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks);
				pw[i*vecSize+j].data[2] = QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks);
				pw[i*vecSize+j].data[3] = QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks);
			}
		}
		for (j = 0; j < vecSize; j++)
		{
			pw[i*vecSize+j].data[0] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[1] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[2] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[3] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks) & 0x00ffffff) + 0x80000000);
		}
		algo.func(pw, hash);
		buffer[0] = 0;
		for (j = 0; j < vecSize; j++)
		{
			for (k = 0; k < 4; k++)
			{
				sprintf(tmpBuffer, "%08x%08x%08x%08x\n", TO_BIG_ENDIAN_32(hash[0*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[1*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[2*vecSize+j].data[k]), TO_BIG_ENDIAN_32(hash[3*vecSize+j].data[k]));
				strcat(buffer, tmpBuffer);
			}
		}
		delete [] deleteMe;
		if (memcmp(buffer,
			"f7d8fd784c89aab673221f68116ebf99\n"
			"71653305007ebcd1de0e35847159db5c\n"
			"4dc44b8b63c6df88a357c23e5d10827f\n"
			"34079f3e2107b345a8edcb45f792990c\n"
			"6199e0c914ed72d9e241f14af23bfced\n"
			"d701ffef5652242e44063fb756ce83c0\n"
			"eacd3437aa279b388bbba248e1718e4b\n"
			"e2d85f090f15aca855997fb21c28e6b3\n"
			"9782b274050f87a363769adbfa96133a\n"
			"1b1f646bdd48cf45a18e3747ac769bf3\n"
			"c2301f1f2f25ad87df34b1ed7cda9400\n"
			"cdc6b2296f9d68a8a0291005009945b0\n"
			"2f370ae2f3194c2781e92d77831affc2\n"
			"e6c928a7f79789e5f17eb41328119ad9\n"
			"1b1dc568253fe19f621e2c05d3cccfc2\n"
			"a69d95c6646465adbbb267213ca7d97a\n"
			"e4982a2fae7cf77d02b71ad36ad5b93d\n"
			"6fffe8350d304f98179562f3d51c4617\n"
			"6b43f795e62919ba915067bceeea880e\n"
			"1dd9c50cd0afa4d18d862ba74b1db1c9\n", strlen(buffer)) == 0)
		{
			printf("PASS\n");
		}
		else
		{
			printf("FAIL\n");
		}
		ks = new KeySpace(subKeySpaces, algo.charaterEncoding);
		benchmark(*ks, algo, 0);
		delete ks;
	}


	// *****************
	// *** Test SHA1 ***
	// *****************
	printf("Testing SHA1...               ");
	fflush(stdout);
	if (HashFactory::getHashInstance("sha1", 0, 23, algo))
	{
		printf("No SHA1 algorithm\n");
	}
	else
	{
		vecSize = (algo.hashesPerCall + 3) / 4;
		numPwBlocks = algo.numPwBlocks;
		numHashBlocks = (algo.len + 3) / 4 + algo.numExtraHashBlocks;
		allocatePwHash(&pw, &hash, &deleteMe, vecSize, numPwBlocks, numHashBlocks);
		for (i = 0; i < vecSize; i++)
		{
			pw[0*vecSize+i].data[0] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[1] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[2] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[3] = numPwBlocks*4-1;
		}
		for (i = 1; i < numPwBlocks; i++)
		{
			for (j = 0; j < vecSize; j++)
			{
				pw[i*vecSize+j].data[0] = QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks);
				pw[i*vecSize+j].data[1] = QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks);
				pw[i*vecSize+j].data[2] = QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks);
				pw[i*vecSize+j].data[3] = QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks);
			}
		}
		for (j = 0; j < vecSize; j++)
		{
			pw[i*vecSize+j].data[0] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[1] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[2] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[3] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks) & 0x00ffffff) + 0x80000000);
		}
		algo.func(pw, hash);
		buffer[0] = 0;
		for (j = 0; j < vecSize; j++)
		{
			for (k = 0; k < 4; k++)
			{
				sprintf(tmpBuffer, "%08x%08x%08x%08x%08x\n", FROM_BIG_ENDIAN_32(hash[0*vecSize+j].data[k]), FROM_BIG_ENDIAN_32(hash[1*vecSize+j].data[k]), FROM_BIG_ENDIAN_32(hash[2*vecSize+j].data[k]), FROM_BIG_ENDIAN_32(hash[3*vecSize+j].data[k]), FROM_BIG_ENDIAN_32(hash[4*vecSize+j].data[k]));
				strcat(buffer, tmpBuffer);
			}
		}
		delete [] deleteMe;
		if (memcmp(buffer,
			"8c3358b4b67e3d189f633762a89f1849cebf6f12\n"
			"1f89c90d6804dc10e4b43dc300ccf18b5e653fdb\n"
			"45f423c0da309b7c000a0b384b48e60c7df156ad\n"
			"fb4fabdc45283ac0353fdaa96b981f6e4d19ee1f\n"
			"013306e48722dae0054997edb090d26b075569fc\n"
			"d612db4dda5657a3c06831f3c4e677d30f39951e\n"
			"4430d72594342a4c197d822702ca2d6fbfb6dc83\n"
			"3ebfda053bf3dde0ac8f4d54162b701fddf55f7b\n"
			"75f01b698f5cae2c142648ff37c7e34ee5b114c0\n"
			"a73a47c9cd13a2ec92109724c89c5240baa22407\n"
			"0e41f6c8934a712852bf7c157cb96e314297b29b\n"
			"d7146ca43ee0202e7d0447ab6e54dd45be022dd7\n", strlen(buffer)) == 0)
		{
			printf("PASS\n");
		}
		else
		{
			printf("FAIL\n");
		}
		ks = new KeySpace(subKeySpaces, algo.charaterEncoding);
		benchmark(*ks, algo, 0);
		delete ks;
	}


	// **********************
	// *** Test MySQLSHA1 ***
	// **********************
	printf("Testing MySQLSHA1/MySQL5...   ");
	fflush(stdout);
	if (HashFactory::getHashInstance("mysqlsha1", 0, 23, algo))
	{
		printf("No MySQLSHA1 algorithm\n");
	}
	else
	{
		vecSize = (algo.hashesPerCall + 3) / 4;
		numPwBlocks = algo.numPwBlocks;
		numHashBlocks = (algo.len + 3) / 4 + algo.numExtraHashBlocks;
		allocatePwHash(&pw, &hash, &deleteMe, vecSize, numPwBlocks, numHashBlocks);
		for (i = 0; i < vecSize; i++)
		{
			pw[0*vecSize+i].data[0] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[1] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[2] = numPwBlocks*4-1;
			pw[0*vecSize+i].data[3] = numPwBlocks*4-1;
		}
		for (i = 1; i < numPwBlocks; i++)
		{
			for (j = 0; j < vecSize; j++)
			{
				pw[i*vecSize+j].data[0] = QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks);
				pw[i*vecSize+j].data[1] = QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks);
				pw[i*vecSize+j].data[2] = QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks);
				pw[i*vecSize+j].data[3] = QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks);
			}
		}
		for (j = 0; j < vecSize; j++)
		{
			pw[i*vecSize+j].data[0] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+0*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[1] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+1*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[2] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+2*numPwBlocks) & 0x00ffffff) + 0x80000000);
			pw[i*vecSize+j].data[3] = TO_LITTLE_ENDIAN_32((QUAD(' ' + (i-1)+4*numPwBlocks*j+3*numPwBlocks) & 0x00ffffff) + 0x80000000);
		}
		algo.func(pw, hash);
		buffer[0] = 0;
		for (j = 0; j < vecSize; j++)
		{
			for (k = 0; k < 4; k++)
			{
				sprintf(tmpBuffer, "%08x%08x%08x%08x%08x\n", FROM_BIG_ENDIAN_32(hash[0*vecSize+j].data[k]), FROM_BIG_ENDIAN_32(hash[1*vecSize+j].data[k]), FROM_BIG_ENDIAN_32(hash[2*vecSize+j].data[k]), FROM_BIG_ENDIAN_32(hash[3*vecSize+j].data[k]), FROM_BIG_ENDIAN_32(hash[4*vecSize+j].data[k]));
				strcat(buffer, tmpBuffer);
			}
		}
		delete [] deleteMe;
		if (memcmp(buffer,
			"015cf6e2ab9de87b25dd23ba268efd075d1fecf4\n"
			"2cc929d88dfde3f1aac2765ddece5dbc2d0053a9\n"
			"a95e67d54fdb4ac4ffc3e561c1dfea10cf74e25e\n"
			"66fc09fb0afa13f03b727977c1e79e4ac3a83c30\n"
			"b406dcd753208b8e444c1d2a6a97c4b85f2faec6\n"
			"d2b9004c216f605d502ae5f2f9d29a78c8ad9c78\n"
			"68ee9650efcf80319086e1a1ae1937ce30eaa163\n"
			"c29808f3d1dff993a6ed2d032bc29af5cfa2b6d5\n"
			"bd0271ffd9969d434e9a503e6cff70b8774aee33\n"
			"2c267c7df2968e4bc7eac49074f1c8ae733d9fba\n"
			"c11f371ff0628821cc27d064483247f9712fc3f2\n"
			"e4adb009482ec75d02b4070575df57978c7efe58\n", strlen(buffer)) == 0)
		{
			printf("PASS\n");
		}
		else
		{
			printf("FAIL\n");
		}
		ks = new KeySpace(subKeySpaces, algo.charaterEncoding);
		benchmark(*ks, algo, 0);
		delete ks;
	}
#ifdef _WIN32
	getchar();
#endif
	return 0;
}

void benchmark(KeySpace &ks, hashAlgorithm hashAlgo, uint32_t tableIndex)
{
	uint64u pwNum;
	uint64_t keySpace;
	double d;
	vector4 *pw, *hash, *deleteMe;
	uint64_t *pwNums;
	uint32_t a, b, count = 1, countTmp, vecSize = (hashAlgo.hashesPerCall + 3) / 4;
	TIMER_TYPE t1, t2;

	pwNums = new uint64_t[hashAlgo.hashesPerCall];
	allocatePwHash(&pw, &hash, &deleteMe, vecSize, hashAlgo.numPwBlocks, (hashAlgo.len + 3) / 4 + hashAlgo.numExtraHashBlocks);

	keySpace = ks.getKeySpace();
	tableIndex <<= 16;
	ks.reset(0);
	ks.next(pw, hashAlgo.hashesPerCall, vecSize);

	// Bench link speed
	do
	{
		TIMER_FUNC(t1);
		for (a = 0; a < count; a++)
		{
			hashAlgo.func(pw, hash);
			for (b = 0; b < hashAlgo.hashesPerCall; b++)
			{
				pwNum.lo32.u32 = FROM_LITTLE_ENDIAN_32(hash[0].data[b]);
				pwNum.hi32.u32 = FROM_LITTLE_ENDIAN_32(hash[vecSize].data[b]);
				pwNums[b] = (pwNum.u64 + (tableIndex + a)) % keySpace;
			}
			ks.get(pw, hashAlgo.hashesPerCall, vecSize, pwNums);
		}
		TIMER_FUNC(t2);
		d = TIMER_DIFF(t1,t2);
		if (d <= .1)
		{
			count *= 10;
		}
		else
		{
			count = (uint32_t) ((double) count / d) + 1;
		}
	} while (d < .9);
	printf("\t%0.2f MLinks/s", (double) hashAlgo.hashesPerCall * count / 1000000.0 / d);
	fflush(stdout);

	// Bench reduction function speed
	countTmp = count;
	do
	{
		TIMER_FUNC(t1);
		for (a = 0; a < count; a++)
		{
			for (b = 0; b < hashAlgo.hashesPerCall; b++)
			{
				pwNum.lo32.u32 = FROM_LITTLE_ENDIAN_32(hash[0].data[b]);
				pwNum.hi32.u32 = FROM_LITTLE_ENDIAN_32(hash[vecSize].data[b]);
				pwNums[b] = (pwNum.u64 + (tableIndex + a)) % keySpace;
			}
			ks.get(pw, hashAlgo.hashesPerCall, vecSize, pwNums);
		}
		TIMER_FUNC(t2);
		d = TIMER_DIFF(t1,t2);
		if (d <= .1)
		{
			count *= 10;
		}
		else
		{
			count = (uint32_t) ((double) count / d) + 1;
		}
	} while (d < .9);
	printf("  %0.2f MReduxes/s", (double) hashAlgo.hashesPerCall * count / 1000000.0 / d);
	fflush(stdout);

	// Bench hash function speed
	count = countTmp;
	do
	{
		TIMER_FUNC(t1);
		for (a = 0; a < count; a++)
		{
			hashAlgo.func(pw, hash);
		}
		TIMER_FUNC(t2);
		d = TIMER_DIFF(t1,t2);
		if (d <= .1)
		{
			count *= 10;
		}
		else
		{
			count = (uint32_t) ((double) count / d) + 1;
		}
	} while (d < .9);
	printf("  %0.2f MHashes/s\n", (double) hashAlgo.hashesPerCall * count / 1000000.0 / d);

	delete [] pwNums;
	delete [] deleteMe;
}
